# -*- coding: utf-8 -*-
# Generated by Django 1.11.11 on 2018-04-19 17:34
from __future__ import unicode_literals

from django.db import migrations, models
import trionyx.trionyx.models


class Migration(migrations.Migration):

    dependencies = [
        ('testblog', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='post',
            name='price',
            field=trionyx.models.PriceField(decimal_places=4, default=0.0, max_digits=11),
        ),
        migrations.AddField(
            model_name='post',
            name='status',
            field=models.IntegerField(choices=[(1, 'New'), (2, 'Draft'), (3, 'Published')], default=1),
        ),
    ]
