{% extends "trionyx/base.html" %}

{% block title %}500 error{% endblock %}
{% block page_title %}500 error{% endblock %}

{% block content %}
    <div class="error-page">
        <h2 class="headline text-red">500</h2>

        <div class="error-content">
          <h3><i class="fa fa-warning text-red"></i> Oops! Something went wrong.</h3>

          <p>
              We will work on fixing that right away. Meanwhile, you may <a href="/">return to dashboard</a>.
          </p>
        </div>
      </div>
{% endblock %}