<table class="table table-condensed">
    <thead>
        <tr>
            {% for field in component.get_fields %}
                <th {% if field.width %}style="width: {{ field.width }};"{% endif %}>
                    {{ field.label }}
                </th>
            {% endfor %}
        </tr>
    </thead>
    <tbody>
        {% for row in component.get_rendered_objects %}
            <tr>
                {% for column in row %}
                   <td>{{ column.value|safe }}</td>
                {% endfor %}
            </tr>
        {% endfor %}
  </tbody>
</table>